﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Joseph Helm and Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion


using System.Collections.Generic;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.PackingEncoding.PackingSequence {
  [Item("Stochastic Insertion Move Generator", "Generates a number of possible Sequence moves from a given Sequence vector.")]
  [StorableClass]
  public class StochasticInsertionMoveGenerator : InsertionMoveGenerator, IMultiMoveGenerator, IStochasticOperator {
    public ILookupParameter<IRandom> RandomParameter {
      get { return (ILookupParameter<IRandom>)Parameters["Random"]; }
    }
    public IValueLookupParameter<IntValue> SampleSizeParameter {
      get { return (IValueLookupParameter<IntValue>)Parameters["SampleSize"]; }
    }

    public IntValue SampleSize {
      get { return SampleSizeParameter.Value; }
      set { SampleSizeParameter.Value = value; }
    }

    [StorableConstructor]
    protected StochasticInsertionMoveGenerator(bool deserializing) : base(deserializing) { }
    protected StochasticInsertionMoveGenerator(StochasticInsertionMoveGenerator original, Cloner cloner) : base(original, cloner) { }
    public StochasticInsertionMoveGenerator()
      : base() {
      Parameters.Add(new LookupParameter<IRandom>("Random", "The random number generator."));
      Parameters.Add(new ValueLookupParameter<IntValue>("SampleSize", "The number of moves to generate."));
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new StochasticInsertionMoveGenerator(this, cloner);
    }

    public static InsertionMove[] Apply(PackingSequenceEncoding PackingSequence, IRandom random, int sampleSize) {

      int nrOfBins = 0;
      foreach (int binNr in PackingSequence.PackingSequence) {
        if (binNr > nrOfBins)
          nrOfBins = binNr;
      }
      nrOfBins++;
      var generatedMoves = new List<InsertionMove>();
      var moves = new List<InsertionMove>(ExhaustiveInsertionMoveGenerator.Generate(PackingSequence, nrOfBins));
      int moveCount = moves.Count;
      for (int i = 0; i < sampleSize; i++) {
        generatedMoves.Add(moves[random.Next(moveCount)]);
      }

      return generatedMoves.ToArray();
    }

    protected override InsertionMove[] GenerateMoves(PackingSequenceEncoding SwapSequence) {
      IRandom random = RandomParameter.ActualValue;
      return Apply(SwapSequence, random, SampleSizeParameter.ActualValue.Value);
    }
  }
}
